﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Model.Extend;
using REBORN.Common;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using Property.Module.FixedAssets;
using System;
using System.Collections.Generic;

namespace Property.WebApi.Controllers.User
{
    public class SuppliesController : PApiController
    {
        readonly SuppliesModule suppliesModule = new SuppliesModule();


        /// <summary>
        /// 获取物料档案分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetMaterialPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Material_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Material_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetMaterialPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                x.Units,
                x.ImageList,
                x.Remark,
                x.SupplierName,
                x.Barcode,
                x.ReservedStock,
                x.SafetyStock,
                x.SuppliesModel,
                x.SuppliesNum,
                x.InventoryNum,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 设置物料档案
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetMaterialInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Material_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Material_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入物料名称");
            }
            if ((demodel.CategoryId ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择物料分类");
            }
            if (string.IsNullOrEmpty(demodel.BrandName))
            {
                return ApiResult.ParamIsNull("请输入品牌信息");
            }
            if (!demodel.ReservedStock.HasValue)
            {
                return ApiResult.ParamIsNull("请输入预留库存数量");
            }
            if (!demodel.SafetyStock.HasValue)
            {
                return ApiResult.ParamIsNull("请输入安全库存数量");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = suppliesModule.SetMaterialInfo(demodel);            
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除物料档案
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelMaterialInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int MaterialId = parms.GetInt("MaterialId", 0);
            if (MaterialId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.DelMaterialInfo(MaterialId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取仓库分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetWareHousePageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetWareHousePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取所有仓库列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetWareHouseList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);            
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetWareHouseList(demodel);

            var robj = list.Select(x => new
            {
                x.Id,
                x.Name
            });
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 设置仓库信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetWareHouseInfo() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入物料仓库名称");
            }
            

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = suppliesModule.SetWareHouseInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除物料仓库
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelWareHouseInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int WareHouseId = parms.GetInt("WareHouseId", 0);
            if (WareHouseId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.DelWareHouseInfo(WareHouseId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取入库单列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetStockInPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetStockInPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.StockInState,
                StockInStateStr=x.StockInState.GetEnumName(),
                StockInDate= x.StockInDate.HasValue?x.StockInDate.Value.ToString("yyyy-MM-dd"):"",
                x.StockInNum,
                x.WareHouseName,
                x.WarehouseId,
                x.Remark,
                x.Money,
                x.SupplierName,
                DetailList=x.DetailList.Select(z=>new { 
                    z.Id,
                    z.CategoryName,
                    z.UnitPrice,
                    z.Money,
                    z.Number,
                    z.SuppliesName,
                    z.SuppliesNum,
                    z.BrandName,
                    z.SuppliesModel,
                    z.SuppliesId,
                    z.Units,
                    z.Remark
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 入库单下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetStockInPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"入库状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"入库编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"入库时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"供应商名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单价"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetStockInPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList) {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.StockInState.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.StockInNum){ Rowspan=Count},
                                    new ExcelColumn(value:item.StockInDate.HasValue?item.StockInDate.Value.ToString("yyyy-MM-dd"):""){ Rowspan=Count},
                                    new ExcelColumn(value:item.WareHouseName){ Rowspan=Count},
                                    new ExcelColumn(value:item.SupplierName){ Rowspan=Count},
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.UnitPrice??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Money??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        else {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.UnitPrice??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Money??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }                        
                        Num++;
                    }
                }
                #endregion                
                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材入库单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材入库单.xls");
            }

        }

        /// <summary>
        /// 设置入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetStockInInfo() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            if (demodel.WarehouseId<=0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any()) {
                return ApiResult.ParamIsNull("请选择物料");
            }
            if (!demodel.StockInDate.HasValue) {
                return ApiResult.ParamIsNull("请输入入库日期");
            }
            foreach (var item in demodel.DetailList) {
                if (item.SuppliesId <= 0) {
                    return ApiResult.ParamIsNull("请传递正确的物料id");
                }
                if (!item.Money.HasValue) {
                    return ApiResult.ParamIsNull("请输入入库金额");
                }
                if (item.Number <= 0) {
                    return ApiResult.ParamIsNull("请输入入库数量");
                }
            }
            if (demodel.DetailList.Any())
            {
                var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                foreach (var item in slist) {
                    if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1) {
                        return ApiResult.Failed("每种耗材只能选择一个");
                    }
                }
            }
            else {
                return ApiResult.Failed("请选择耗材");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;

            bool flag = suppliesModule.SetStockInInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelStockInInfo() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model = suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("入库单已被删除");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockInWarting)
            {
                return ApiResult.Failed("入库单已完成，无法删除");
            }

            bool flag = suppliesModule.DelStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 取消入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelStockInInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            //判断状态  是否是待入库
            var model = suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("入库单已被删除");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockIn)
            {
                return ApiResult.Failed("入库单未入库，无法取消");
            }
            //验证库存是否足够减少
            if (!suppliesModule.ValidateStockInDel(StockInId))
            {
                return ApiResult.Failed("库存不足，请核实后再试");
            }

            bool flag = suppliesModule.CancelStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 恢复入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult RecoverStockInInfo() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model= suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1) {
                return ApiResult.Failed("入库单已被删除，无法恢复");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockInWarting) {
                return ApiResult.Failed("入库单已完成，无法修改");
            }

            bool flag = suppliesModule.RecoverStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取出库单分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetStockOutPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Supplies_StockOut_Extend> list = suppliesModule.GetStockOutPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.StockOutState,
                StockOutStateStr = x.StockOutState.GetEnumName(),
                StockOutDate = x.StockOutDate.HasValue ? x.StockOutDate.Value.ToString("yyyy-MM-dd") : "",
                x.StockOutNum,
                x.WareHouseName,
                x.WarehouseId,
                x.Remark,
                x.EmployeeId,
                EmName = CacheManager.User.UserReidsCache.GetEmployee(x.EmployeeId)?.EmName ?? "",
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.Number,
                    z.SuppliesName,
                    z.SuppliesNum,
                    z.BrandName,
                    z.SuppliesModel,
                    z.SuppliesId,
                    z.Units,
                    z.Remark,
                    z.InventoryNum
                }),                
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 出库单下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetStockOutPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"出库状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"出库编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"出库时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"库存"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                List<RB_Supplies_StockOut_Extend> list = suppliesModule.GetStockOutPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.StockOutState.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.StockOutNum){ Rowspan=Count},
                                    new ExcelColumn(value:item.StockOutDate.HasValue?item.StockOutDate.Value.ToString("yyyy-MM-dd"):""){ Rowspan=Count},
                                    new ExcelColumn(value:item.WareHouseName){ Rowspan=Count},
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.InventoryNum??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.InventoryNum??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion                
                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材出库单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材出库单.xls");
            }

        }

        /// <summary>
        /// 保存出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetStockOutInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            if (demodel.WarehouseId <= 0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择物料");
            }
            if (!demodel.StockOutDate.HasValue)
            {
                return ApiResult.ParamIsNull("请输入入库日期");
            }
            if (demodel.EmployeeId <= 0) {
                return ApiResult.ParamIsNull("请选择领用人");
            }
            foreach (var item in demodel.DetailList)
            {
                if (item.SuppliesId <= 0)
                {
                    return ApiResult.ParamIsNull("请传递正确的物料id");
                }                
                if (item.Number <= 0)
                {
                    return ApiResult.ParamIsNull("请输入入库数量");
                }
            }
            if (demodel.DetailList.Any())
            {
                var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                foreach (var item in slist)
                {
                    if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1)
                    {
                        return ApiResult.Failed("每种耗材只能选择一个");
                    }
                }
            }
            else
            {
                return ApiResult.Failed("请选择耗材");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;//暂时这么赋值   看是否需要前端自行选择
            demodel.DepartmentId = emodel.RB_Department_Id;

            //验证库存是否足够
            if (!suppliesModule.ValidateStockOutInsert(demodel))
            {
                return ApiResult.Failed("库存不足无法出库，请核实后再试");
            }

            bool flag = suppliesModule.SetStockOutInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelStockOutInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockOutId = parms.GetInt("StockOutId", 0);
            if (StockOutId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model = suppliesModule.GetStockOutInfo(StockOutId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("出库单已被删除");
            }
            if (model.StockOutState != REBORN.Common.Enum.StockOutStatusEnum.StockOutCancel)
            {
                return ApiResult.Failed("出库单未取消，无法删除");
            }

            bool flag = suppliesModule.DelStockOutInfo(StockOutId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 取消出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelStockOutInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockOutId = parms.GetInt("StockOutId", 0);
            if (StockOutId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.CancelStockOutInfo(StockOutId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取库存分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetInventoryPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetInventoryPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Number,
                x.Money,
                UnitPrice = x.Number == 0 ? 0 : Math.Round((x.Money ?? 0) / (x.Number ?? 1), 2, MidpointRounding.AwayFromZero),
                SuppliesName = x.SModel.Name,
                x.SuppliesId,
                x.WarehouseId,
                x.SModel.SuppliesNum,
                x.SModel.CategoryName,
                x.SModel.SuppliesModel,
                x.SModel.BrandName,
                x.SModel.Units,
                x.SModel.SafetyStock,
                x.SModel.ReservedStock,
                WareHouseName = x.WModel.Name
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 物料库存统计下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetInventoryPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单位"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"安全库存"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预留库存"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单价"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"库存数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetInventoryPageList(1, 999999, demodel, out long count);

                #region 组装数据
                int TotalNum = 0;
                foreach (var item in list)
                {
                    TotalNum += (item.Number ?? 0);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.SModel.SuppliesNum){ },
                            new ExcelColumn(value:item.SModel.Name){ },
                            new ExcelColumn(value:item.SModel.CategoryName){ },
                            new ExcelColumn(value:item.SModel.SuppliesModel){ },
                            new ExcelColumn(value:item.WModel.Name){ },
                            new ExcelColumn(value:item.SModel.BrandName){ },
                            new ExcelColumn(value:item.SModel.Units){ },
                            new ExcelColumn(value:(item.SModel.SafetyStock??0).ToString()){ },
                            new ExcelColumn(value:(item.SModel.ReservedStock??0).ToString()){ },
                            new ExcelColumn(value:(item.Number == 0 ? 0 : Math.Round((item.Money ?? 0) / (item.Number ?? 1), 2, MidpointRounding.AwayFromZero)).ToString()){ },
                            new ExcelColumn(value:(item.Number??0).ToString()){ },
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计（共"+list.Count()+"条）"){ Colspan=10},
                            new ExcelColumn(value:TotalNum.ToString("#0")){},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "物料库存统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "物料库存统计.xls");
            }

        }

        /// <summary>
        /// 获取库存明细分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetInventoryDetailPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetInventoryDetailPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Type,
                TypeName = x.Type == 1 ? "增加" : "减少",
                x.Number,
                x.Money,
                x.Description,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.CreateBy)?.EmName ?? "",
                UpdateDate = x.CreateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 耗材领用查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetRecipientsQueryPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockOutDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOutDetail_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetRecipientsQueryPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);

            RB_Supplies_StockOutDetail_Extend Total = suppliesModule.GetRecipientsQueryTotal(demodel);//统计
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Number,
                x.Money,
                x.BranchName,
                x.DepartmentName,
                x.EmName,
                x.CategoryName,
                x.SuppliesName,
                x.StockOutNum,
                StockOutDate = x.StockOutDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", new {
                pagelist,
                Total=new {
                    Total.Number,
                    Total.Money
                }
            });
        }

        /// <summary>
        /// 耗材领用统计下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetRecipientsQueryPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOutDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOutDetail_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"领取单号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"公司名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"部门名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领取日期"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领取数量"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetRecipientsQueryPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int TotalNum = 0;
                decimal TotalMoney = 0;
                foreach (var item in list)
                {
                    TotalNum += (item.Number ?? 0);
                    TotalMoney += (item.Money ?? 0);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.StockOutNum){ },
                            new ExcelColumn(value:item.BranchName){ },
                            new ExcelColumn(value:item.DepartmentName){ },
                            new ExcelColumn(value:item.EmName){ },
                            new ExcelColumn(value:item.StockOutDate.Value.ToString("yyyy-MM-dd")){ },
                            new ExcelColumn(value:item.SuppliesName){ },
                            new ExcelColumn(value:item.CategoryName){ },
                            new ExcelColumn(value:(item.Number??0).ToString()){ },
                            new ExcelColumn(value:(item.Money??0).ToString()){ }
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计（共"+list.Count()+"条）"){ Colspan=8},
                            new ExcelColumn(value:TotalNum.ToString("#0")){},
                            new ExcelColumn(value:TotalMoney.ToString("#0.00")){},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材领用统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材领用统计.xls");
            }

        }

        #region 耗材申请日期

        /// <summary>
        /// 获取耗材申请设置分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetApplyForSetPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Applyfor_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Applyfor_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Supplies_Applyfor_Extend> list = suppliesModule.GetApplyForSetPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.BranchId,
                BranchName = x.BranchId==-1?"通用": CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                x.Enabled,
                WeekdayList = x.WeekdayList.OrderBy(x=>x.Id).Select(x => new
                {
                    x.Id,
                    x.ApplyforId,
                    x.Weekday,
                    x.StartTime,
                    x.EndTime
                }),
                SpecialList = x.SpecialList.Select(x => new
                {
                    x.Id,
                    x.ApplyforId,
                    Date = x.Date.Value.ToString("yyyy-MM-dd"),
                    x.StartTime,
                    x.EndTime
                }),
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 新增耗材申请设置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetApplyForInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Applyfor_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Applyfor_Extend>(requestParm.msg.ToString());

            if ((demodel.BranchId ?? -1) < -1) {
                return ApiResult.ParamIsNull("请选择公司别");
            }
            if (demodel.WeekdayList == null || !demodel.WeekdayList.Any()) {
                return ApiResult.ParamIsNull("请选择工作日");
            }
            foreach (var item in demodel.WeekdayList) {
                if (string.IsNullOrEmpty(item.Weekday)) {
                    return ApiResult.ParamIsNull("工作日设置有误");
                }
                if (string.IsNullOrEmpty(item.StartTime)) {
                    return ApiResult.ParamIsNull("工作日时间设置有误");
                }
                if (string.IsNullOrEmpty(item.EndTime)){
                    return ApiResult.ParamIsNull("工作日时间设置有误");
                }
            }
            if (demodel.SpecialList != null && demodel.SpecialList.Any()) {
                foreach (var item in demodel.SpecialList) {
                    if (!item.Date.HasValue)
                    {
                        return ApiResult.ParamIsNull("请选择特殊日期");
                    }
                    if (string.IsNullOrEmpty(item.StartTime))
                    {
                        return ApiResult.ParamIsNull("特殊日期时间设置有误");
                    }
                    if (string.IsNullOrEmpty(item.EndTime))
                    {
                        return ApiResult.ParamIsNull("特殊日期时间设置有误");
                    }
                }
            }

            //验证公司别是否重复
            if (suppliesModule.ValidateBranch(demodel, userInfo)) {
                return ApiResult.Failed("该公司已存在记录，无法新增");
            }

            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;

            bool flag = suppliesModule.SetApplyForInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 设置申请状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetApplyForStatus() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ApplyforId = parms.GetInt("ApplyforId", 0);
            if (ApplyforId == 0) {
                return ApiResult.Failed("请传递id");
            }

            bool flag = suppliesModule.SetApplyForStatus(ApplyforId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取耗材申请设置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetApplyForInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            
            var Model = suppliesModule.GetApplyForInfo(userInfo);
            if (Model.Id == -1)
            {
                return ApiResult.Success("", new { Status = 1, Model = "" });
            }
            else {
                return ApiResult.Success("", new
                {
                    Status = 2,
                    Model = new
                    {
                        Model.Id,
                        Model.BranchId,
                        BranchName = Model.BranchId==-1?"通用": CacheManager.User.BranchReidsCache.GetBranch(Model.BranchId)?.BName ?? "",
                        WeekdayList = Model.WeekdayList.OrderBy(x=>x.Id).Select(x => new
                        {
                            x.Weekday,
                            x.StartTime,
                            x.EndTime
                        }),
                        SpecialList = Model.SpecialList.OrderBy(x=>x.Date).Select(x => new
                        {
                            Date = x.Date.Value.ToString("yyyy-MM-dd"),
                            x.StartTime,
                            x.EndTime
                        })
                    }
                });
            }
        }

        /// <summary>
        /// 是否可以申请
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetCanApplyFor()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            var Model = suppliesModule.GetApplyForInfo(userInfo);
            if (Model.Id == -1)
            {
                return ApiResult.Success();
            }
            else
            {
                if (Model.SpecialList != null && Model.SpecialList.Any()) {
                    var smodel = Model.SpecialList.Where(x => x.Date == Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd"))).FirstOrDefault();
                    if (smodel != null)
                    {
                        smodel.StartTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + smodel.StartTime;
                        smodel.EndTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + smodel.EndTime;
                        if (Convert.ToDateTime(smodel.StartTime) <= DateTime.Now && DateTime.Now <= Convert.ToDateTime(smodel.EndTime))
                        {
                            return ApiResult.Success();
                        }
                        else {
                            return ApiResult.Failed();
                        }
                    }
                }
                int Week = (int)DateTime.Now.DayOfWeek;
                string WeekDay = "";
                switch (Week) {
                    case 0:
                        WeekDay = "周日";break;
                    case 1:
                        WeekDay = "周一"; break;
                    case 2:
                        WeekDay = "周二"; break;
                    case 3:
                        WeekDay = "周三"; break;
                    case 4:
                        WeekDay = "周四"; break;
                    case 5:
                        WeekDay = "周五"; break;
                    case 6:
                        WeekDay = "周六"; break;
                }
                var wmodel = Model.WeekdayList.Where(x => x.Weekday == WeekDay).FirstOrDefault();
                if (wmodel == null) {
                    return ApiResult.Failed();
                }
                wmodel.StartTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + wmodel.StartTime;
                wmodel.EndTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + wmodel.EndTime;
                if (Convert.ToDateTime(wmodel.StartTime) <= DateTime.Now && DateTime.Now <= Convert.ToDateTime(wmodel.EndTime))
                {
                    return ApiResult.Success();
                }
                else {
                    return ApiResult.Failed();
                }

            }
        }

        #endregion

    }
}
